from .staxing import Stax
from .helpers import StaxHelper, Admin, Student, Teacher
from .assignment import Assignment

if __name__ == '__main__':
    a = Stax
    b = StaxHelper
    c = Assignment
    d = Admin
    e = Student
    f = Teacher
